#!/usr/bin/env python3

# Command line interface to start crawlers

import argparse
import os
import signal
import sys
import threading
import validators
from crawler import StreamCrawler

import crawler.trafikverket

VERSION = "1.0.0"

parser = argparse.ArgumentParser(description="Crawler for Trafikverket's Trafikinfo API - version " + VERSION)

# Add the arguments
parser.add_argument('objecttype',
                       metavar='objecttype',
                       type=str,
                       help='the API objecttype to crawl',
                       choices=["TrainAnnouncement", "TrainMessage"])

parser.add_argument('-s', '--stream', 
                        action='store_true',
                        help='use streaming in real-time instead of daily crawling'
                        )

parser.add_argument('-c', '--compress', 
                        action='store_true',
                        help='write zip compressed csv files'
                        )

parser.add_argument('-f', '--force', 
                        action='store_true',
                        help='force new connection instead of reconnecting'
                        )

parser.add_argument('-l', '--locationSignature', 
                        metavar="locationSignature",
                        type=str,
                        help='the location signature of the station to crawl data for',
                        required=True
                        )

parser.add_argument('-o', '--output', 
                metavar="path",
                type=str,
                help='the output folder',
                default=None
                )

parser.add_argument("-w", '--webhook',
                metavar="webhookurl",
                type=str,
                help="Webhook URL to trigger when warnings or exceptions occur",
                default=None,
                required=False)

parser.add_argument("-a", '--authkey',
                metavar="authkey",
                type=str,
                help="The API authkey to use as obtained from https://api.trafikinfo.trafikverket.se/Account/Register",
                default=crawler.trafikverket.DEFAULT_AUTHKEY,
                required=False)

parser.add_argument("-v", '--verbose',
                action='store_true',
                help="Prints extra debug information")

parser.add_argument("-q", '--query',
                action='store_true',
                help="Print query only instead of starting a crawler")

# Validate arguments
args = parser.parse_args()

if args.output is None:
    #not specified, use default
    if args.stream:
        args.output = os.path.join(".", "stream", args.objecttype, args.locationSignature)
    else:
        args.output = os.path.join(".", "daily", args.objecttype, args.locationSignature)
else:
    #specified, check if exists
    if not os.path.isdir(args.output):
        print("The output path {output} does not exist and will be created.".format(output=args.output))

        try:
            os.makedirs(args.output)
            print("The output path {output} has been created.".format(output=args.output))
        except Exception as e:
            print("The output path {output} could not be created: {exception}".format(output=args.output, exception=e))
            exit(-1)

if not args.webhook is None and not validators.url(args.webhook):
    print("Webhook is not a valid URL.")
    exit(-1)

if args.force and not args.stream:
    print("Forcing a new connection is only available when streaming")
    exit(-1)

# Prepare crawler
if args.objecttype == "TrainAnnouncement" and args.stream:
    c = crawler.trafikverket.StreamTrainAnnouncementCrawler(
            args.output, args.authkey, 
            locationSignature=args.locationSignature,
            version=VERSION
        )
elif args.objecttype == "TrainAnnouncement" and not args.stream:
    c = crawler.trafikverket.DailyTrainAnnouncementCrawler(
            args.output, args.authkey, 
            locationSignature=args.locationSignature,
            version=VERSION
        )
elif args.objecttype == "TrainMessage" and args.stream:
    c = crawler.trafikverket.StreamTrainMessageCrawler(
            args.output, args.authkey, 
            locationSignature=args.locationSignature,
            version=VERSION
        )
elif args.objecttype == "TrainMessage" and not args.stream:
    c = crawler.trafikverket.DailyTrainMessageCrawler(
            args.output, args.authkey, 
            locationSignature=args.locationSignature,
            version=VERSION
        )

if not args.webhook is None:
    c.message_webhook_url = args.webhook

c.compress = args.compress

if c.compress:
    print("Compression not supported yet.")

if args.verbose:
    c.DEBUG = True
    print("Verbose output enabled.")

if args.query:
    print(c._get_query())
    sys.exit(0)

# Print status
def print_status():
    if not c.messages.empty():
        print("")
        while not c.messages.empty():
            print(c.messages.get())


    if not c.stop_request.is_set() and isinstance(c, StreamCrawler) and not c.connected.is_set():
        print("{name}\tconnecting\t{warnings} warnings\t{exceptions} exceptions".format(
            name=c.path, writes=c.writes, warnings=c.warnings, exceptions=c.exceptions),end='\r')
    elif not c.stop_request.is_set() and c.running.is_set():
        print("{name}\t{writes} writes\t{warnings} warnings\t{exceptions} exceptions".format(
            name=c.path, writes=c.writes, warnings=c.warnings, exceptions=c.exceptions),end='\r')
    elif c.stop_request.is_set() and c.running.is_set():
        print("{name}\tstopping\t{warnings} warnings\t{exceptions} exceptions".format(
            name=c.path, writes=c.writes, warnings=c.warnings, exceptions=c.exceptions),end='\r')   
    else:
        print("{name}\tnot running\t{warnings} warnings\t{exceptions} exceptions".format(
            name=c.path, writes=c.writes, warnings=c.warnings, exceptions=c.exceptions),end='\r')

    if c.running.is_set() or not c.stop_request.is_set():
        threading.Timer(1, print_status).start()
    else:
        sys.exit(0)

print_status()

#Log command run
c.log_status("Command run: {cmd}".format(cmd=" ".join(sys.argv)))

# Start crawling
if args.stream and args.force:
    c.start(force_new_connection=True)
else:
    c.start()

def signal_handler(sig, frame):
    c.messages.put("Crawler stopping...")
    c.stop()
    if c.join(wait_seconds=3):
        print("\nCrawler stopped.")
    sys.exit(0)

signal.signal(signal.SIGINT, signal_handler)
c.messages.put("∞ Crawler running. Press CTRL-C to stop.")
